@@ -95,7 +95,8 @@ namespace xt
     namespace detail
     {
         template <class E, class F>
-        std::ostream& xoutput(std::ostream& out, const E& e, slice_vector& slices, F& printer, std::size_t blanks,
+        std::ostream& xoutput(std::ostream& out, const E& e,
+                              xstrided_slice_vector& slices, F& printer, std::size_t blanks,
                               std::streamsize element_width, std::size_t edgeitems, std::size_t line_width)
         {
             using size_type = typename E::size_type;
@@ -115,7 +116,7 @@ namespace xt
                 size_type line_lim = static_cast<size_type>(std::floor(line_width / ewp2));
 
                 out << '{';
-                for (; i != view.shape()[0] - 1; ++i)
+                for (; i != size_type(view.shape()[0] - 1); ++i)
                 {
                     if (edgeitems && size_type(view.shape()[0]) > (edgeitems * 2) && i == edgeitems)
                     {
@@ -126,7 +127,7 @@ namespace xt
                             out << std::endl
                                 << indents;
                         }
-                        i = view.shape()[0] - edgeitems;
+                        i = size_type(view.shape()[0]) - edgeitems;
                     }
                     if (view.dimension() == 1 && line_lim != 0 && elems_on_line >= line_lim)
                     {
@@ -162,7 +163,7 @@ namespace xt
         }
 
         template <class F, class E>
-        static void recurser_run(F& fn, const E& e, slice_vector& slices, std::size_t lim = 0)
+        static void recurser_run(F& fn, const E& e, xstrided_slice_vector& slices, std::size_t lim = 0)
         {
             using size_type = typename E::size_type;
             const auto view = strided_view(e, slices);
@@ -173,11 +174,11 @@ namespace xt
             else
             {
                 size_type i = 0;
-                for (; i != view.shape()[0] - 1; ++i)
+                for (; i != static_cast<size_type>(view.shape()[0] - 1); ++i)
                 {
                     if (lim && size_type(view.shape()[0]) > (lim * 2) && i == lim)
                     {
-                        i = view.shape()[0] - lim;
+                        i = static_cast<size_type>(view.shape()[0]) - lim;
                     }
                     slices.push_back(static_cast<int>(i));
                     recurser_run(fn, e, slices, lim);
@@ -612,7 +613,7 @@ namespace xt
 
         detail::printer<E> p(precision);
 
-        slice_vector sv;
+        xstrided_slice_vector sv;
         detail::recurser_run(p, d, sv, lim);
         p.init();
         sv.clear();
