@@ -83,7 +83,6 @@ namespace xt
 
         using reduce_functor_type = typename std::decay_t<F>::reduce_functor_type;
         using init_functor_type = typename std::decay_t<F>::init_functor_type;
-        using merge_functor_type = typename std::decay_t<F>::merge_functor_type;
 
         using expr_value_type = typename std::decay_t<E>::value_type;
 
@@ -95,7 +94,7 @@ namespace xt
         auto init_fct = std::get<1>(f);
         auto merge_fct = std::get<2>(f);
 
-        shape_type result_shape;
+        shape_type result_shape{};
         resize_container(result_shape, e.dimension() - axes.size());
 
         dynamic_shape<std::size_t> iter_shape = xtl::forward_sequence<dynamic_shape<std::size_t>>(e.shape());
@@ -136,10 +135,17 @@ namespace xt
 
         result.resize(result_shape, e.layout());
 
-        std::size_t ax_idx = (e.layout() == layout_type::row_major) ? axes.size() - 1 : 0;
-        std::size_t inner_loop_size = e.strides()[axes[ax_idx]];
-        std::size_t inner_stride = e.strides()[axes[ax_idx]];
-        std::size_t outer_loop_size = e.shape()[axes[ax_idx]];
+        std::size_t leading_ax = axes[(e.layout() == layout_type::row_major) ? axes.size() - 1 : 0];
+        auto strides_finder = e.strides().begin() + static_cast<std::ptrdiff_t>(leading_ax);
+        // The computed strides contain "0" where the shape is 1 -- therefore find the next none-zero number
+        std::size_t inner_stride = *strides_finder;
+        while (inner_stride == 0)
+        {
+            (e.layout() == layout_type::row_major) ? --strides_finder : ++strides_finder;
+            inner_stride = *strides_finder;
+        }
+        std::size_t inner_loop_size = inner_stride;
+        std::size_t outer_loop_size = e.shape()[leading_ax];
 
         // The following code merges reduction axes "at the end" (or the beginning for col_major)
         // together by increasing the size of the outer loop where appropriate
@@ -825,7 +831,6 @@ namespace xt
         auto stepper = const_stepper(*this, 0);
         size_type dim = 0;
         // drop left most elements
-        using difference_type = typename std::iterator_traits<It>::difference_type;
         auto size = std::ptrdiff_t(dimension()) - std::distance(first, last);
         auto begin = first - size;
         while (begin != last)
