@@ -347,6 +347,11 @@ namespace xt
         void to_begin() noexcept;
         void to_end(layout_type l) noexcept;
 
+        template <class R>
+        R step_simd();
+
+        value_type step_leading();
+
     private:
 
         storage_type* p_c;
@@ -1013,6 +1018,20 @@ namespace xt
     {
     }
 
+    template <bool is_const, class CT>
+    template <class R>
+    inline R xscalar_stepper<is_const, CT>::step_simd()
+    {
+        return R(p_c->operator()());
+    }
+
+    template <bool is_const, class CT>
+    inline auto xscalar_stepper<is_const, CT>::step_leading()
+        -> value_type
+    {
+        return p_c->operator()();
+    }
+
     /**********************************
      * xdummy_iterator implementation *
      **********************************/
